
#include "ResultViewModel.h"
#include <QtAlgorithms>

ResultViewModel::ResultViewModel(QObject *parent) : QAbstractItemModel(parent) {
}

ResultViewModel::~ResultViewModel() {

}

QVariant ResultViewModel::headerData (int section, Qt::Orientation orientation, int role) const {
	
	if(role == Qt::DisplayRole && orientation == Qt::Horizontal) {
		switch(section) {
		case 0:
			return "Block";
		case 1:
			return "Size";
		case 2:
			return "Type";
		default:
			return QVariant();
		}	
	}
	
	return QVariant();

}

QVariant ResultViewModel::data(const QModelIndex &index, int role) const {

	if (!index.isValid())
		return QVariant();

	if (role != Qt::DisplayRole)
		return QVariant();

	switch(index.column()) {
	case 0:
		return QString().sprintf("%08x", m_Results[index.row()].block);
	case 1:
		return QString().sprintf("%08x", m_Results[index.row()].size);
	case 2:
		return m_Results[index.row()].type;
	default:
		return QVariant();
	}
}

void ResultViewModel::addResult(const Result &r) {
	m_Results.push_back(r);
	reset();
}

void ResultViewModel::clearResults() {
	m_Results.clear();
	reset();
}

QModelIndex ResultViewModel::index(int row, int column, const QModelIndex &parent) const {

	Q_UNUSED(parent);

	if(row >= m_Results.size()) {
		return QModelIndex();
	}

	if(column >= 3) {
		return QModelIndex();
	}
	
	return createIndex(row, column, const_cast<Result *>(&m_Results[row]));
}

QModelIndex ResultViewModel::parent(const QModelIndex &index) const {
	Q_UNUSED(index);
	return QModelIndex(); 
}

int ResultViewModel::rowCount(const QModelIndex &parent) const {
	Q_UNUSED(parent);
	return m_Results.size();
}

int ResultViewModel::columnCount(const QModelIndex &parent) const {
	Q_UNUSED(parent);
	return 3;
}

void ResultViewModel::sort(int column, Qt::SortOrder order) {

	if(order == Qt::AscendingOrder) {
		switch(column) {
		case 0:
			qSort(m_Results.begin(), m_Results.end(), blockLess);
			break;
		case 1:
			qSort(m_Results.begin(), m_Results.end(), SizeLess);
			break;
		case 2:
			qSort(m_Results.begin(), m_Results.end(), TypeLess);
			break;
		}
	} else {
		switch(column) {
		case 0:
			qSort(m_Results.begin(), m_Results.end(), blockGreater);
			break;
		case 1:
			qSort(m_Results.begin(), m_Results.end(), SizeGreater);
			break;
		case 2:
			qSort(m_Results.begin(), m_Results.end(), TypeGreater);
			break;
		}
	}
	emit dataChanged(createIndex(0, 0, 0), createIndex(-1, -1, 0));
}

