/*
Copyright (C) 2006 Evan Teran
                   eteran@alum.rit.edu

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#ifndef DIALOGOPENFILES_20070423_H_
#define DIALOGOPENFILES_20070423_H_

#include "ui_dialogopenfiles.h"
#include "types.h"
#include <QString>
#include <QFile>
#include <QTextStream>

class DialogOpenFiles : public QDialog {
	Q_OBJECT
	
public:
	DialogOpenFiles(QWidget *parent = 0);

public slots:
	void on_btnRefresh_clicked();

private:
	virtual void showEvent(QShowEvent *event);
	
private:
	static QString fileType(const QString &filename);
	static QString processSocketTCP(QString &symlink);
	static QString processSocketUDP(QString &symlink);
	static QString processSocketUNIX(QString &symlink);
	
	
	static bool TCPSocketProcessor(QString &symlink, int socketNumber, const QStringList &lst);
	static bool UDPSocketProcessor(QString &symlink, int socketNumber, const QStringList &lst);
	static bool UNIXSocketProcessor(QString &symlink, int socketNumber, const QStringList &lst);
	
	
	typedef bool (*processor_t)(QString &s, int socketNumber, const QStringList &lst);
	
	// template to reduce writing same thing many times
	template<processor_t P>
	static QString processSocketFile(const QString &filename, QString &symlink, int socketNumber) {
		
		QFile net(filename);
		net.open(QIODevice::ReadOnly | QIODevice::Text);
		if(net.isOpen()) {
			QTextStream in(&net);
			QString line;
			
			// ditch first line, it is just table headings
			line = in.readLine();

			// read in the first line we care about
			line = in.readLine();

			// a "null" string means end of file (but not an empty string!)
			while (!line.isNull()) {

				QString lline(line);
				lline.replace(":", " ");
				const QStringList lst = lline.split(" ", QString::SkipEmptyParts);

				if((*P)(symlink, socketNumber, lst)) {
					break;
				}

				line = in.readLine();
			}
		}
		return symlink;
	}
	
public:
	 Ui::DialogOpenFiles ui;
};

#endif
