
#include "CommentServer.h"
#include "DisassemblyLib.h"
#include "Debugger.h"
#include "DebuggerCoreInterface.h"
#include <QString>

CommentServer::CommentServer() {
}

void CommentServer::setComment(BaseTypes::address_t address, const QString &comment) {
	Q_UNUSED(address);
	Q_UNUSED(comment);
}

QString CommentServer::resolveFunctionCall(BaseTypes::address_t address) const {
	QString ret;
	
	uint8_t buffer[MAX_INSN_SIZE];
	BaseTypes::reg_t value;
	
	if(Debugger::debuggerBase->readBytes(address, &value, sizeof(value))) {
		// TODO: portability warning, assumes a call is 5 bytes
		if(Debugger::debuggerBase->readBytes(value - 6, buffer, sizeof(buffer))) {


			for(int i = 4; i >= 0; --i) {
				Instruction insn(buffer + i, sizeof(buffer) - i, 0);
				if(insn.valid() && insn.type() == insn_call) {
					const QString symname = Debugger::findFunctionSymbol(value, "");
					if(!symname.isEmpty()) {
						ret = QString().sprintf("Return to %08x <%s>", value, qPrintable(symname));
					} else {
						ret = QString().sprintf("Return to %08x", value);
					}
					break;
				}
			}
		}
	}
	return ret;
}

QString CommentServer::resolveAsciiString(BaseTypes::address_t address) const {
	QString ret;
	QString temp;
	
	BaseTypes::reg_t value;
	if(Debugger::debuggerBase->readBytes(address, &value, sizeof(value))) {
		if(Debugger::getASCIIStringAtAddress(value, temp, Debugger::config()->minAsciiStringLength, 256)) {
			ret = "ASCII \"" + temp + "\"";
		}
	}
	return ret;
}

QString CommentServer::getComment(BaseTypes::address_t address) const {

	QString ret;
	
	ret = resolveFunctionCall(address);
	if(ret.isEmpty()) {
		ret = resolveAsciiString(address);
	}
	

	return ret;
}

