/*
Copyright (C) 2006 Evan Teran
                   eteran@alum.rit.edu

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#include "State.h"
#include "Util.h"
#include <QString>

//------------------------------------------------------------------------------
// Name: State()
// Desc: constructor
//------------------------------------------------------------------------------
State::State() : ebx(0), ecx(0), edx(0), esi(0), edi(0), ebp(0), eax(0), esp(0),
		_cs(0), _ds(0), _es(0), _fs(0), _gs(0), _ss(0), eip(0), eflags(0), 
		orig_eax(0) {
}

//------------------------------------------------------------------------------
// Name: State()
// Desc: constructor
//------------------------------------------------------------------------------
State::State(const State &o) : ebx(o.ebx), ecx(o.ecx), edx(o.edx), esi(o.esi), 
		edi(o.edi), ebp(o.ebp), eax(o.eax), esp(o.esp),
		_cs(o._cs), _ds(o._ds), _es(o._es), _fs(o._fs), _gs(o._gs), _ss(o._ss), eip(o.eip), eflags(o.eflags), 
		orig_eax(o.orig_eax) {

}

//------------------------------------------------------------------------------
// Name: clear()
// Desc: 
//------------------------------------------------------------------------------
void State::clear() {
	eax = 0;
	ebx = 0;
	ecx = 0;
	edx = 0;
	esi = 0;
	edi = 0;
	ebp = 0;
	esp = 0;
	_cs = 0;
	_ds = 0;
	_es = 0;
	_fs = 0;
	_gs = 0;
	_ss = 0;
	eip = 0;
	eflags = 0;
	orig_eax = 0;
}

//------------------------------------------------------------------------------
// Name: value(const QString &reg, bool *ok)
// Desc: a function to return the value of a register based on it's name
//------------------------------------------------------------------------------
BaseTypes::reg_t State::value(const QString &reg, bool *ok) const {
	BaseTypes::reg_t ret = 0;
	setBool(ok, false);

	const QString lreg(reg.toLower());

	if(lreg == "eax") { ret = eax; setBool(ok, true); }
	else if(lreg == "ebx") { ret = ebx; setBool(ok, true); }
	else if(lreg == "ecx") { ret = ecx; setBool(ok, true); }
	else if(lreg == "edx") { ret = edx; setBool(ok, true); }
	else if(lreg == "ebp") { ret = ebp; setBool(ok, true); }
	else if(lreg == "esp") { ret = esp; setBool(ok, true); }
	else if(lreg == "esi") { ret = esi; setBool(ok, true); }
	else if(lreg == "edi") { ret = edi; setBool(ok, true); }
	else if(lreg == "eip") { ret = eip; setBool(ok, true); }
	else if(lreg == "ax") { ret = eax & 0xffff; setBool(ok, true); }
	else if(lreg == "bx") { ret = ebx & 0xffff; setBool(ok, true); }
	else if(lreg == "cx") { ret = ecx & 0xffff; setBool(ok, true); }
	else if(lreg == "dx") { ret = edx & 0xffff; setBool(ok, true); }
	else if(lreg == "bp") { ret = ebp & 0xffff; setBool(ok, true); }
	else if(lreg == "sp") { ret = esp & 0xffff; setBool(ok, true); }
	else if(lreg == "si") { ret = esi & 0xffff; setBool(ok, true); }
	else if(lreg == "di") { ret = edi & 0xffff; setBool(ok, true); }
	else if(lreg == "al") { ret = eax & 0xff; setBool(ok, true); }
	else if(lreg == "bl") { ret = ebx & 0xff; setBool(ok, true); }
	else if(lreg == "cl") { ret = ecx & 0xff; setBool(ok, true); }
	else if(lreg == "dl") { ret = edx & 0xff; setBool(ok, true); }
	else if(lreg == "ah") { ret = (eax >> 8) & 0xff; setBool(ok, true); }
	else if(lreg == "bh") { ret = (ebx >> 8) & 0xff; setBool(ok, true); }
	else if(lreg == "ch") { ret = (ecx >> 8) & 0xff; setBool(ok, true); }
	else if(lreg == "dh") { ret = (edx >> 8) & 0xff; setBool(ok, true); }

	return ret;
}
