/*
Copyright (C) 2006 Evan Teran
                   eteran@alum.rit.edu

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/


#ifndef QHEXVIEW_20060506_H_
#define QHEXVIEW_20060506_H_

#include <QAbstractScrollArea>
#include <QByteArray>
#include <QMap>
#include <QString>
#include "types.h"


class QMenu;
class ByteStream;
class CommentServerInterface;

class QHexView : public QAbstractScrollArea {
	Q_OBJECT
	
public:
	typedef ByteStream C;
	typedef BaseTypes::address_t address_t;
	
public:
	QHexView(QWidget * parent = 0);
	virtual ~QHexView();
	
public:
	void setCommentServer(CommentServerInterface *p);
	CommentServerInterface * commentServer() const;
		
protected:
	virtual void paintEvent(QPaintEvent * event);
	virtual void resizeEvent(QResizeEvent * event);
	virtual void mousePressEvent(QMouseEvent * event);
	virtual void mouseMoveEvent(QMouseEvent * event);
	virtual void mouseReleaseEvent(QMouseEvent * event);
	virtual void keyPressEvent(QKeyEvent * event);
	virtual void mouseDoubleClickEvent(QMouseEvent * event);
	virtual void contextMenuEvent(QContextMenuEvent * event);

public slots:
	virtual void setShowAddress(bool);
	virtual void setShowAsciiDump(bool);
	virtual void setShowHexDump(bool);
	virtual void setShowComments(bool);
	//virtual void setLineColor(QColor);
	//virtual void setAddressColor(QColor);
	virtual void setWordWidth(int);
	virtual void setRowWidth(int);
	virtual void setFont(const QFont &font);
		
public:
	address_t addressOffset() const;
	bool showHexDump() const;
	bool showAddress() const;
	bool showAsciiDump() const;
	bool showComments() const;
	QColor lineColor() const;
	QColor addressColor() const;
	int wordWidth() const;
	int rowWidth() const;
	
private:
	int m_RowWidth;			// amount of "words" per row
	int m_WordWidth;		// size of a "word" in bytes
	QColor m_AddressColor;	// colour of the address in display
	QColor m_LineColor;		// colour of the lines in display
	bool m_ShowHex;			// should we show the hex display?
	bool m_ShowAscii;		// should we show the ascii display?
	bool m_ShowAddress;		// should we show the address display?
	bool m_ShowComments;

public:
	void setData(C *d);
	void setAddressOffset(address_t offset);
	void scrollTo(unsigned int offset);
	
	address_t selectedBytesAddress() const;
	unsigned int selectedBytesSize() const;
	QByteArray selectedBytes() const;
	QByteArray allBytes() const;
	QMenu *createStandardContextMenu();

public slots:
	void clear();
	void selectAll();
	void deselect();
	bool hasSelectedText() const;
	void mnuSetFont();
	void mnuCopy();
	
	void mnuSetRowWidth1();
	void mnuSetRowWidth2();
	void mnuSetRowWidth4();
	void mnuSetRowWidth8();
	void mnuSetRowWidth16();	
	void mnuSetWordWidth1();
	void mnuSetWordWidth2();
	void mnuSetWordWidth4();
	void mnuSetWordWidth8();

private:
	void updateScrollbars();
	
	bool isSelected(int index) const;
	static bool isPrintable(unsigned int ch);
	bool isInViewableArea(int index) const;
	
	int pixelToWord(unsigned int x, unsigned int y) const;
	
	unsigned int charsPerWord() const;
	unsigned int hexDumpLeft() const;
	unsigned int asciiDumpLeft() const;
	unsigned int commentLeft() const;
	unsigned int addressLen() const;
	unsigned int line1() const;
	unsigned int line2() const;
	unsigned int line3() const;

	unsigned int bytesPerRow() const;
	
	int dataSize() const;
	
	void drawAsciiDump(QPainter &painter, unsigned int offset, unsigned int row) const;
	void drawHexDump(QPainter &painter, unsigned int offset, unsigned int row, int &wordCount) const;
	void drawComments(QPainter &painter, unsigned int offset, unsigned int row) const;
	
	QString formatAddress(address_t address);
	
private:
	static bool addToggleActionToMenu(QMenu *menu, const QString &caption, bool checked, QObject *reciever, const char *slot);
	
private:
	address_t m_Origin;
	address_t m_AddressOffset;		// this is the offset that our base address is relative to
	int m_SelectionStart;			// index of first selected word (or -1)
	int m_SelectionEnd;				// index of last selected word (or -1)
	int m_FontWidth;				// width of a character in this font
	int m_FontHeight;				// height of a character in this font
	C *m_Data;						// the current data
	
	enum {
		Highlighting_None,
		Highlighting_Data,
		Highlighting_Ascii
	} m_Highlighting;
	
	QColor m_EvenWord;
	QColor m_OddWord;
	QColor m_PrintableText;
	QColor m_NonPrintableText;
	QColor m_MarkedText;
	QColor m_MarkedTextBackground;
	
	char m_UnprintableChar;

	bool m_ShowLine1;
	bool m_ShowLine2;
	bool m_ShowLine3;
	bool m_ShowAddressSeparator;	// should we show ':' character in address to seperate high/low portions
	char m_AddressFormatString[32];
	
	CommentServerInterface *m_CommentServer;
};

#endif
